import logging

from django.core.management.base import BaseCommand
from django.utils import timezone

from openlxp_xia.management.utils.xia_internal import (
    dict_flatten, required_recommended_logs)
from openlxp_xia.management.utils.xss_client import (
    get_required_fields_for_validation, get_source_validation_schema)
from openlxp_xia.models import MetadataLedger

logger = logging.getLogger('dict_config_logger')


def get_source_metadata_for_validation():
    """Retrieving source metadata from MetadataLedger that needs to be
        validated"""
    logger.info(
        "Accessing source metadata from MetadataLedger to be validated")
    source_data_dict = \
        MetadataLedger.objects.values('source_metadata_key_hash',
                                      'source_metadata').filter(
            source_metadata_validation_status='',
            record_lifecycle_status='Active').exclude(
            source_metadata_extraction_date=None)

    return source_data_dict


def store_source_metadata_validation_status(source_data_dict,
                                            key_value_hash, validation_result,
                                            record_status_result,
                                            source_metadata):
    """Storing validation result in MetadataLedger"""

    if record_status_result == 'Active':
        source_data_dict.filter(
            source_metadata_key_hash=key_value_hash).update(
            source_metadata=source_metadata,
            source_metadata_validation_status=validation_result,
            source_metadata_validation_date=timezone.now(),
            record_lifecycle_status=record_status_result
        )
    else:
        source_data_dict.filter(
            source_metadata_key_hash=key_value_hash).update(
            source_metadata=source_metadata,
            source_metadata_validation_status=validation_result,
            source_metadata_validation_date=timezone.now(),
            record_lifecycle_status=record_status_result,
            metadata_record_inactivation_date=timezone.now()
        )


def validate_source_using_key(source_data_dict, required_column_list,
                              recommended_column_list):
    """Validating source data against required & recommended column names"""

    logger.info("Validating and updating records in MetadataLedger table for "
                "Source data")
    len_source_metadata = len(source_data_dict)
    for ind in range(len_source_metadata):
        # Updating default validation for all records
        validation_result = 'Y'
        record_status_result = 'Active'

        # flattened source data created for reference
        flattened_source_data = dict_flatten(source_data_dict[ind]
                                             ['source_metadata'],
                                             required_column_list)
        # validate for required values in data
        for item in required_column_list:
            # update validation and record status for invalid data
            # Log out error for missing required values
            if item in flattened_source_data:
                if not flattened_source_data[item]:
                    validation_result = 'N'
                    required_recommended_logs(ind, "Required", item)
            else:
                validation_result = 'N'
                required_recommended_logs(ind, "Required", item)

        # validate for recommended values in data
        for item in recommended_column_list:
            # Log out warning for missing recommended values
            if item in flattened_source_data:
                if not flattened_source_data[item]:
                    required_recommended_logs(ind, "Recommended", item)
            else:
                required_recommended_logs(ind, "Recommended", item)
        # assigning key hash value for source metadata
        key_value_hash = source_data_dict[ind]['source_metadata_key_hash']
        # Calling function to update validation status
        store_source_metadata_validation_status(source_data_dict,
                                                key_value_hash,
                                                validation_result,
                                                record_status_result,
                                                source_data_dict[ind]
                                                ['source_metadata'])


class Command(BaseCommand):
    """Django command to validate source data"""

    def handle(self, *args, **options):
        """
            Source data is validated and stored in metadataLedger
        """
        schema_data_dict = get_source_validation_schema()
        required_column_list, recommended_column_list = \
            get_required_fields_for_validation(schema_data_dict)
        source_data_dict = get_source_metadata_for_validation()
        validate_source_using_key(source_data_dict, required_column_list,
                                  recommended_column_list)

        logger.info(
            'MetadataLedger updated with source metadata validation status')



import logging
from unittest.mock import patch

from ddt import ddt
from django.test import tag
from django.utils import timezone

from openlxp_xia.management.commands.load_supplemental_metadata import (
    load_supplemental_metadata_to_xis, post_supplemental_metadata_to_xis,
    rename_supplemental_metadata_fields)
from openlxp_xia.management.commands.load_target_metadata import (
    get_records_to_load_into_xis, post_data_to_xis,
    rename_metadata_ledger_fields)
from openlxp_xia.management.commands.transform_source_metadata import (
    create_supplemental_metadata, create_target_metadata_dict,
    get_metadata_fields_to_overwrite, get_source_metadata_for_transformation,
    overwrite_append_metadata, overwrite_metadata_field,
    transform_source_using_key, type_checking_target_metadata)
from openlxp_xia.management.commands.validate_source_metadata import (
    get_source_metadata_for_validation, validate_source_using_key)
from openlxp_xia.management.commands.validate_target_metadata import (
    get_target_metadata_for_validation, update_previous_instance_in_metadata,
    validate_target_using_key)
from openlxp_xia.models import (MetadataFieldOverwrite, MetadataLedger,
                                SupplementalLedger, XIAConfiguration,
                                XISConfiguration)

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('unit')
@ddt
class CommandTests(TestSetUp):

    # Test cases for validate_source_metadata

    def test_get_source_metadata_for_validation(self):
        """Test to Retrieving source metadata from MetadataLedger that needs
        to be validated"""
        with patch('openlxp_xia.management.commands.'
                   'validate_source_metadata.MetadataLedger.objects') \
                as meta_obj:
            meta_ledger = MetadataLedger.objects.values(
                source_metadata=self.source_metadata).filter(
                source_metadata_validation_status='',
                record_lifecycle_status='Active').exclude(
                source_metadata_extraction_date=None)
            meta_obj.first.return_value = meta_ledger
            return_from_function = get_source_metadata_for_validation()
            self.assertEqual(meta_obj.first.return_value, return_from_function)

    def test_validate_source_using_key_more_than_one(self):
        """Test to Validating source data against required & recommended
        column names for more than one row"""
        data = [{'source_metadata_key_hash': 123,
                 'source_metadata': self.source_metadata},
                {'source_metadata_key_hash': 123,
                 'source_metadata': self.source_metadata}]

        recommended_column_name = []
        with patch('openlxp_xia.management.commands.'
                   'validate_source_metadata'
                   '.store_source_metadata_validation_status',
                   return_value=None) as mock_store_source_valid_status:
            validate_source_using_key(data, self.test_required_column_names,
                                      recommended_column_name)
            self.assertEqual(
                mock_store_source_valid_status.call_count, 2)

    def test_validate_source_using_key_more_than_zero(self):
        """Test to Validating source data against required/ recommended column
            names with no data"""
        data = []
        recommended_column_name = []
        with patch('openlxp_xia.management.commands.'
                   'validate_source_metadata'
                   '.store_source_metadata_validation_status',
                   return_value=None) as mock_store_source_valid_status:
            validate_source_using_key(data, self.test_required_column_names,
                                      recommended_column_name)
            self.assertEqual(
                mock_store_source_valid_status.call_count, 0)

    # Test cases for transform_source_metadata

    def test_type_checking_target_metadata(self):
        """"Test function for type checking and explicit type conversion of
        metadata """
        target_metadata = {
            0: self.target_metadata}
        target_metadata = type_checking_target_metadata(1, target_metadata,
                                                        self.expected_datatype)
        self.assertIsInstance(target_metadata[0]['General_Information'][
                                  "EndDate"], str)

    def test_get_source_metadata_for_transformation(self):
        """Test to Retrieving Source metadata from MetadataLedger that needs
        to be transformed"""
        with patch('openlxp_xia.management.commands.'
                   'transform_source_metadata'
                   '.MetadataLedger.objects') as meta_obj:
            target_data_dict = MetadataLedger.objects.values(
                source_data_dict=self.source_metadata).filter(
                source_metadata_validation_status='Y',
                record_lifecycle_status='Active').exclude(
                source_metadata_validation_date=None)
            meta_obj.first.return_value = target_data_dict
            return_from_function = get_source_metadata_for_transformation()
            self.assertEqual(meta_obj.first.return_value,
                             return_from_function)

    def test_create_supplemental_data(self):
        """Test to check creation of supplemental data from source data"""

        supplemental_data = \
            create_supplemental_metadata(self.test_metadata_column_list,
                                         self.
                                         source_metadata_with_supplemental)
        self.assertEqual(supplemental_data, self.supplemental_data)

    def test_create_target_metadata_dict(self):
        """Test for a function to replace and transform source data to target
        data for using target mapping schema"""
        expected_data_dict = {0: self.target_metadata}
        with patch('openlxp_xia.management.utils.xia_internal.'
                   'dict_flatten', return_value=self.source_metadata), \
                patch('openlxp_xia.management.commands.'
                      'transform_source_metadata.create_supplemental_metadata',
                      return_value=None):
            result_data_dict, supplemental_data = create_target_metadata_dict(
                1, self.source_target_mapping, self.source_metadata,
                self.test_required_column_names, self.expected_datatype)
            self.assertEqual(result_data_dict[0]['Course'].get('CourseCode'),
                             expected_data_dict[0]['Course'].get('CourseCode'))
            self.assertEqual(
                result_data_dict[0]['Course'].get('CourseProviderName'),
                expected_data_dict[0]['Course'].get('CourseProviderName'))

    def test_transform_source_using_key_more_zero(self):
        """Test for transforming source data using target metadata schema
        with no data"""
        data = []
        with patch('openlxp_xia.management.utils.xia_internal'
                   '.get_target_metadata_key_value',
                   return_value=None), \
                patch('openlxp_xia.management.commands.'
                      'transform_source_metadata'
                      '.store_transformed_source_metadata',
                      return_value=None) as mock_store_transformed_source:
            mock_store_transformed_source.return_value = \
                mock_store_transformed_source
            mock_store_transformed_source.exclude.return_value = \
                mock_store_transformed_source
            mock_store_transformed_source.filter.side_effect = [
                mock_store_transformed_source, mock_store_transformed_source]

            transform_source_using_key(data, self.source_target_mapping,
                                       self.test_required_column_names,
                                       self.expected_datatype)

            self.assertEqual(
                mock_store_transformed_source.call_count, 0)

    def test_transform_source_using_key_more_than_one(self):
        """Test for transforming source data using target metadata schema for
        more than one row"""
        data = [{0: self.source_metadata},
                {1: self.source_metadata}]
        with patch('openlxp_xia.management.utils.xia_internal'
                   '.get_target_metadata_key_value',
                   return_value=None), \
                patch('openlxp_xia.management.commands.'
                      'transform_source_metadata'
                      '.store_transformed_source_metadata',
                      return_value=None) as mock_store_transformed_source:
            mock_store_transformed_source.return_value = \
                mock_store_transformed_source
            mock_store_transformed_source.exclude.return_value = \
                mock_store_transformed_source
            mock_store_transformed_source.filter.side_effect = [
                mock_store_transformed_source, mock_store_transformed_source]

            transform_source_using_key(data, self.source_target_mapping,
                                       self.test_required_column_names,
                                       self.expected_datatype)

            self.assertEqual(
                mock_store_transformed_source.call_count, 2)

    def test_overwrite_metadata_field(self):
        """Test to overwrite metadata with admin entered values and
        return metadata in dictionary format """

        with patch('openlxp_xia.management.commands.transform_source_metadata'
                   '.get_metadata_fields_to_overwrite') as mock_get_overwrite:
            mock_get_overwrite.return_value = self.metadata_df

            return_val = overwrite_metadata_field(self.metadata_df)
            self.assertIsInstance(return_val, dict)

    def test_get_metadata_fields_to_overwrite(self):
        """Test for looping through fields to be overwrite or appended"""
        with patch('openlxp_xia.management.commands.'
                   'transform_source_metadata.MetadataFieldOverwrite.'
                   'objects') as mock_field, \
                patch('openlxp_xia.management.commands.'
                      'transform_source_metadata.'
                      'overwrite_append_metadata') as mock_overwrite_fun:
            config = \
                [MetadataFieldOverwrite(field_name='column1', overwrite=True,
                                        field_value='value1'),
                 MetadataFieldOverwrite(field_name='column2', overwrite=False,
                                        field_value='value2')]
            mock_field.all.return_value = config
            mock_overwrite_fun.return_value = self.metadata_df

            get_metadata_fields_to_overwrite(self.metadata_df)
            self.assertEqual(mock_overwrite_fun.call_count, 2)

    def test_overwrite_append_metadata(self):
        """test Overwrite & append metadata fields based on overwrite flag """
        return_val = \
            overwrite_append_metadata(self.metadata_df, 'column1', 'value1',
                                      True)

        self.assertEqual(return_val['column1'].all(), 'value1')

    # Test cases for validate_target_metadata

    def test_get_target_metadata_for_validation(self):
        """Test to Retrieving target metadata from MetadataLedger that needs
        to be validated"""
        with patch('openlxp_xia.management.commands.'
                   'validate_target_metadata.MetadataLedger.objects') \
                as meta_obj:
            target_data_dict = MetadataLedger.objects.values(
                target_metadata=self.target_metadata).filter(
                target_metadata_validation_status='',
                record_lifecycle_status='Active').exclude(
                source_metadata_transformation_date=None)
            meta_obj.first.return_value = target_data_dict
            return_from_function = get_target_metadata_for_validation()
            self.assertEqual(meta_obj.first.return_value,
                             return_from_function)

    def test_validate_target_using_key_more_than_one(self):
        """Test to Validating target data against required & recommended
        column names for more than one row"""
        data = [{'target_metadata_key_hash': 123,
                 'target_metadata': self.target_metadata},
                {'target_metadata_key_hash': 123,
                 'target_metadata': self.target_metadata}]
        test_required_column_names = {
            'CourseInstance.EndDate', 'CourseInstance.DeliveryMode',
            'CourseInstance.CourseCode', 'CourseInstance.Instructor',
            'Course.CourseProviderName', 'Course.CourseDescription',
            'Course.CourseShortDescription', 'CourseInstance.StartDate',
            'Course.CourseCode', 'CourseInstance.CourseTitle ',
            'General_Information.StartDate', 'Course.CourseSubjectMatter',
            'General_Information.EndDate', 'Course.CourseTitle',
            'Technical.CourseTitle'}
        recommended_column_name = {'Technical_Information.Thumbnail',
                                   'CourseInstance.Thumbnail'}
        with patch('openlxp_xia.management.commands.'
                   'validate_target_metadata'
                   '.store_target_metadata_validation_status',
                   return_value=None) as mock_store_target_valid_status:
            validate_target_using_key(data, test_required_column_names,
                                      recommended_column_name,
                                      self.expected_datatype)
            self.assertEqual(
                mock_store_target_valid_status.call_count, 2)

    def test_validate_target_using_key_zero(self):
        """Validating target data against required & recommended column names
        with no data"""

        data = []

        with patch('openlxp_xia.management.commands.'
                   'validate_target_metadata'
                   '.store_target_metadata_validation_status',
                   return_value=None) as mock_store_target_valid_status:
            validate_target_using_key(data,
                                      self.test_target_required_column_names,
                                      self.recommended_column_name,
                                      self.expected_datatype)

            self.assertEqual(mock_store_target_valid_status.call_count, 0)

    def test_update_previous_instance_in_metadata(self):
        """test to check Update older instances of record to inactive status"""
        metadata = MetadataLedger(source_metadata=self.source_metadata,
                                  target_metadata=self.target_metadata,
                                  target_metadata_key=self.target_key_value,
                                  target_metadata_key_hash=self.
                                  target_key_value_hash,
                                  source_metadata_key_hash=self.
                                  target_key_value_hash,
                                  target_metadata_hash=self.target_hash_value,
                                  record_lifecycle_status='Active',
                                  target_metadata_validation_date=timezone.
                                  now())
        metadata.save()
        update_previous_instance_in_metadata(self.key_value_hash)

        updated_value = MetadataLedger.objects. \
            get(target_metadata_key_hash=self.target_key_value_hash)
        self.assertEqual(updated_value.record_lifecycle_status, 'Inactive')

    # Test cases for load_target_metadata

    def test_rename_metadata_ledger_fields(self):
        """Test for Renaming XIA column names to match with XIS column names"""
        with patch('openlxp_xia.management.utils.xia_internal'
                   '.get_publisher_detail'), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.XIAConfiguration.objects') as xisCfg:
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xisCfg.first.return_value = xiaConfig
            return_data = rename_metadata_ledger_fields(self.xia_data)
            self.assertEquals(self.xis_expected_data['metadata_hash'],
                              return_data['metadata_hash'])
            self.assertEquals(self.xis_expected_data['metadata_key'],
                              return_data['metadata_key'])
            self.assertEquals(self.xis_expected_data['metadata_key_hash'],
                              return_data['metadata_key_hash'])
            self.assertEquals(self.xis_expected_data['provider_name'],
                              return_data['provider_name'])

    def test_get_records_to_load_into_xis_one_record(self):
        """Test to Retrieve number of Metadata_Ledger records in XIA to load
        into XIS  and calls the post_data_to_xis accordingly"""
        with patch('openlxp_xia.management.commands.'
                   'load_target_metadata.post_data_to_xis', return_value=None
                   )as mock_post_data_to_xis, \
                patch('openlxp_xia.management.commands.load_target_metadata.'
                      'MetadataLedger.objects') as meta_obj:
            meta_data = MetadataLedger(
                record_lifecycle_status='Active',
                source_metadata=self.source_metadata,
                target_metadata=self.target_metadata,
                target_metadata_hash=self.target_hash_value,
                target_metadata_key_hash=self.target_key_value_hash,
                target_metadata_key=self.target_key_value,
                source_metadata_transformation_date=timezone.now(),
                target_metadata_validation_status='Y',
                source_metadata_validation_status='Y',
                target_metadata_transmission_status='Ready')
            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.values.return_value = [meta_data]
            meta_obj.filter.side_effect = [meta_obj, meta_obj]
            get_records_to_load_into_xis()
            self.assertEqual(
                mock_post_data_to_xis.call_count, 1)

    def test_get_records_to_load_into_xis_zero(self):
        """Test to Retrieve number of Metadata_Ledger records in XIA to load
        into XIS  and calls the post_data_to_xis accordingly"""
        with patch(
                'openlxp_xia.management.commands.load_target_metadata'
                '.post_data_to_xis', return_value=None)as \
                mock_post_data_to_xis, \
                patch(
                    'openlxp_xia.management.commands.'
                    'load_target_metadata.MetadataLedger.objects') as meta_obj:
            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.filter.side_effect = [meta_obj, meta_obj]
            get_records_to_load_into_xis()
            self.assertEqual(
                mock_post_data_to_xis.call_count, 0)

    def test_post_data_to_xis_zero(self):
        """Test for POSTing XIA metadata_ledger to XIS metadata_ledger
        when data is not present"""
        data = []
        with patch('openlxp_xia.management.commands'
                   '.load_target_metadata.rename_metadata_ledger_fields',
                   return_value=self.xis_expected_data), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.get_publisher_detail'), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.XIAConfiguration.objects') as xiaCfg, \
                patch('openlxp_xia.management.commands.load_target_metadata.'
                      'MetadataLedger.objects') as meta_obj, \
                patch('requests.post') as response_obj, \
                patch(
                    'openlxp_xia.management.commands.'
                    'load_target_metadata'
                    '.get_records_to_load_into_xis',
                    return_value=None) as mock_check_records_to_load:
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaCfg.first.return_value = xiaConfig
            response_obj.return_value = response_obj
            response_obj.status_code = 201

            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.update.return_value = meta_obj
            meta_obj.filter.side_effect = [meta_obj, meta_obj, meta_obj,
                                           meta_obj]

            post_data_to_xis(data)
            self.assertEqual(response_obj.call_count, 0)
            self.assertEqual(mock_check_records_to_load.call_count, 1)

    def test_post_data_to_xis_more_than_one(self):
        """Test for POSTing XIA metadata_ledger to XIS metadata_ledger
        when more than one rows are passing"""
        data = [self.xia_data,
                self.xia_data]
        with patch(
                'openlxp_xia.management.commands.load_target_metadata'
                '.rename_metadata_ledger_fields',
                return_value=self.xis_expected_data), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.get_publisher_detail'), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.XIAConfiguration.objects') as xiaCfg, \
                patch('openlxp_xia.management.commands.load_target_metadata.'
                      'MetadataLedger.objects') as meta_obj, \
                patch('requests.post') as response_obj, \
                patch('openlxp_xia.management.utils.xis_client.'
                      'XISConfiguration.objects') as xisCfg, \
                patch('openlxp_xia.management.commands.load_target_metadata.'
                      'get_records_to_load_into_xis',
                      return_value=None) as mock_check_records_to_load:
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaCfg.first.return_value = xiaConfig
            xisConfig = XISConfiguration(
                xis_metadata_api_endpoint=self.xis_api_endpoint_url)
            xisCfg.first.return_value = xisConfig
            response_obj.return_value = response_obj
            response_obj.status_code = 201

            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.update.return_value = meta_obj
            meta_obj.filter.side_effect = [meta_obj, meta_obj, meta_obj,
                                           meta_obj]

            post_data_to_xis(data)
            self.assertEqual(response_obj.call_count, 2)
            self.assertEqual(mock_check_records_to_load.call_count, 1)

        # Test cases for load_supplemental_metadata

    def test_rename_supplemental_metadata_fields(self):
        """Test for Renaming XIA column names to match with XIS column names"""
        with patch('openlxp_xia.management.utils.xia_internal'
                   '.get_publisher_detail'), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.XIAConfiguration.objects') as xisCfg:
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xisCfg.first.return_value = xiaConfig
            return_data = rename_supplemental_metadata_fields(
                self.xia_supplemental_data)
            self.assertEquals(
                self.xis_supplemental_expected_data['metadata_hash'],
                return_data['metadata_hash'])
            self.assertEquals(
                self.xis_supplemental_expected_data['metadata_key'],
                return_data['metadata_key'])
            self.assertEquals(
                self.xis_supplemental_expected_data['metadata_key_hash'],
                return_data['metadata_key_hash'])
            self.assertEquals(
                self.xis_supplemental_expected_data['provider_name'],
                return_data['provider_name'])

    def test_load_supplemental_metadata_to_xis_one_record(self):
        """Test to Retrieve number of Metadata_Ledger records in XIA to load
        into XIS  and calls the post_data_to_xis accordingly"""
        with patch(
                'openlxp_xia.management.commands.'
                'load_supplemental_metadata'
                '.post_supplemental_metadata_to_xis', return_value=None)as \
                mock_post_data_to_xis, \
                patch(
                    'openlxp_xia.management.commands.'
                    'load_supplemental_metadata'
                    '.SupplementalLedger.objects') as meta_obj:
            meta_data = SupplementalLedger(
                record_lifecycle_status='Active',
                supplemental_metadata=self.supplemental_data,
                supplemental_metadata_hash=self.target_hash_value,
                supplemental_metadata_key_hash=self.target_key_value_hash,
                supplemental_metadata_key=self.target_key_value,
                supplemental_metadata_transmission_date=timezone.now(),
                supplemental_metadata_transmission_status='Ready')
            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.values.return_value = [meta_data]
            meta_obj.filter.side_effect = [meta_obj, meta_obj]
            load_supplemental_metadata_to_xis()
            self.assertEqual(
                mock_post_data_to_xis.call_count, 1)

    def test_load_supplemental_metadata_to_xis_zero(self):
        """Test to Retrieve number of Metadata_Ledger records in XIA to load
        into XIS  and calls the post_data_to_xis accordingly"""
        with patch(
                'openlxp_xia.management.commands.'
                'load_supplemental_metadata'
                '.post_supplemental_metadata_to_xis', return_value=None)as \
                mock_post_data_to_xis, \
                patch('openlxp_xia.management.commands.'
                      'load_supplemental_metadata.'
                      'SupplementalLedger.objects') as meta_obj:
            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.filter.side_effect = [meta_obj, meta_obj]
            load_supplemental_metadata_to_xis()
            self.assertEqual(
                mock_post_data_to_xis.call_count, 0)

    def test_post_supplemental_metadata_to_xis_zero(self):
        """Test for POSTing XIA metadata_ledger to XIS metadata_ledger
        when data is not present"""
        data = []
        with patch(
                'openlxp_xia.management.commands.'
                'load_supplemental_metadata'
                '.rename_supplemental_metadata_fields',
                return_value=self.xis_expected_data), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.get_publisher_detail'), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.XIAConfiguration.objects') as xiaCfg, \
                patch(
                    'openlxp_xia.management.commands.'
                    'load_supplemental_metadata'
                    '.SupplementalLedger.objects') as meta_obj, \
                patch('requests.post') as response_obj, \
                patch('openlxp_xia.management.commands.'
                      'load_supplemental_metadata.'
                      'load_supplemental_metadata_to_xis',
                      return_value=None) as mock_check_records_to_load:
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaCfg.first.return_value = xiaConfig
            response_obj.return_value = response_obj
            response_obj.status_code = 201

            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.update.return_value = meta_obj
            meta_obj.filter.side_effect = [meta_obj, meta_obj, meta_obj,
                                           meta_obj]

            post_supplemental_metadata_to_xis(data)
            self.assertEqual(response_obj.call_count, 0)
            self.assertEqual(mock_check_records_to_load.call_count, 1)

    def test_post_supplemental_metadata_to_xis_more_than_one(self):
        """Test for POSTing XIA metadata_ledger to XIS metadata_ledger
        when more than one rows are passing"""
        data = [self.xia_supplemental_data,
                self.xia_supplemental_data]
        with patch(
                'openlxp_xia.management.commands.'
                'load_supplemental_metadata'
                '.rename_supplemental_metadata_fields',
                return_value=self.xis_expected_data), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.get_publisher_detail'), \
                patch('openlxp_xia.management.utils.xia_internal'
                      '.XIAConfiguration.objects') as xiaCfg, \
                patch(
                    'openlxp_xia.management.commands.'
                    'load_supplemental_metadata'
                    '.SupplementalLedger.objects') as meta_obj, \
                patch('requests.post') as response_obj, \
                patch('openlxp_xia.management.utils.xis_client'
                      '.XISConfiguration.objects') as xisCfg, \
                patch(
                    'openlxp_xia.management.commands.'
                    'load_supplemental_metadata'
                    '.load_supplemental_metadata_to_xis',
                    return_value=None) as mock_check_records_to_load:
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaCfg.first.return_value = xiaConfig
            xisConfig = \
                XISConfiguration(xis_metadata_api_endpoint=self.
                                 supplemental_api_endpoint)
            xisCfg.first.return_value = xisConfig
            response_obj.return_value = response_obj
            response_obj.status_code = 201

            meta_obj.return_value = meta_obj
            meta_obj.exclude.return_value = meta_obj
            meta_obj.update.return_value = meta_obj
            meta_obj.filter.side_effect = [meta_obj, meta_obj, meta_obj,
                                           meta_obj]

            post_supplemental_metadata_to_xis(data)
            self.assertEqual(response_obj.call_count, 2)
            self.assertEqual(mock_check_records_to_load.call_count, 1)



import logging
from unittest.mock import patch

from ddt import ddt
from django.test import tag
from django.utils import timezone

from openlxp_xia.management.commands.load_target_metadata import (
    post_data_to_xis, rename_metadata_ledger_fields)
from openlxp_xia.management.commands.transform_source_metadata import (
    store_transformed_source_metadata, transform_source_using_key)
from openlxp_xia.management.commands.validate_source_metadata import (
    get_source_metadata_for_validation,
    store_source_metadata_validation_status, validate_source_using_key)
from openlxp_xia.management.commands.validate_target_metadata import (
    get_target_validation_schema, store_target_metadata_validation_status,
    validate_target_using_key)
from openlxp_xia.management.utils.xss_client import read_json_data
from openlxp_xia.models import (MetadataFieldOverwrite, MetadataLedger,
                                SupplementalLedger, XIAConfiguration,
                                XISConfiguration)

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('integration')
@ddt
class CommandIntegration(TestSetUp):
    # globally accessible data sets

    # Test cases for validate_source_metadata

    def test_get_source_metadata_for_validation(self):
        """Test retrieving  source metadata from MetadataLedger that
        needs to be validated"""

        metadata_ledger = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.source_metadata,
            source_metadata_hash=self.hash_value,
            source_metadata_key=self.key_value,
            source_metadata_key_hash=self.key_value_hash,
            source_metadata_extraction_date=timezone.now())
        metadata_ledger.save()
        test_source_data = get_source_metadata_for_validation()
        self.assertTrue(test_source_data)

    def test_validate_source_using_key(self):
        """Test to check validation process for source"""

        recommended_column_name = []
        metadata_ledger = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.source_metadata,
            source_metadata_hash=self.hash_value,
            source_metadata_key=self.key_value,
            source_metadata_key_hash=self.key_value_hash,
            source_metadata_extraction_date=timezone.now())
        metadata_ledger_invalid = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.metadata_invalid,
            source_metadata_hash=self.hash_value_invalid,
            source_metadata_key=self.key_value_invalid,
            source_metadata_key_hash=self.key_value_hash_invalid,
            source_metadata_extraction_date=timezone.now())
        metadata_ledger.save()
        metadata_ledger_invalid.save()
        result_test_query = MetadataLedger.objects. \
            values('source_metadata_key_hash', 'source_metadata')
        validate_source_using_key(result_test_query,
                                  self.test_required_column_names,
                                  recommended_column_name)
        result_query = MetadataLedger.objects. \
            values('source_metadata_validation_status',
                   'record_lifecycle_status'). \
            filter(source_metadata_key=self.key_value).first()
        result_query_invalid = MetadataLedger.objects. \
            values('source_metadata_validation_status',
                   'record_lifecycle_status'). \
            filter(source_metadata_key=self.key_value_invalid).first()
        self.assertEqual('Y',
                         result_query['source_metadata_validation_status'])
        self.assertEqual('Active',
                         result_query['record_lifecycle_status'])
        self.assertEqual('N', result_query_invalid[
            'source_metadata_validation_status'])

    def test_store_source_metadata_validation_status_valid(self):
        """Test to store validation status for valid source metadata
            in metadata ledger """

        metadata_ledger = MetadataLedger(
            source_metadata=self.source_metadata,
            source_metadata_key_hash=self.key_value_hash,
            source_metadata_key=self.key_value)
        metadata_ledger.save()
        store_source_metadata_validation_status(MetadataLedger.objects.
                                                values('source_metadata'),
                                                self.key_value_hash,
                                                'Y', 'Active',
                                                MetadataLedger.objects.
                                                values('source_metadata')
                                                )
        result_query = MetadataLedger.objects.values(
            'source_metadata_validation_status',
            'source_metadata_validation_date',
            'record_lifecycle_status').filter(
            source_metadata_key_hash=self.key_value_hash).first()

        self.assertTrue(result_query.get('source_metadata_validation_date'))
        self.assertEqual("Y", result_query.get(
            'source_metadata_validation_status'))
        self.assertEqual('Active', result_query.get(
            'record_lifecycle_status'))

    def test_store_source_metadata_validation_status_invalid(self):
        """Test to store validation status for invalid source metadata
            in metadata ledger """

        metadata_ledger = MetadataLedger(
            source_metadata=self.source_metadata,
            source_metadata_key_hash=self.key_value_hash,
            source_metadata_key=self.key_value)
        metadata_ledger.save()
        store_source_metadata_validation_status(MetadataLedger.objects.
                                                values('source_metadata'),
                                                self.key_value_hash,
                                                'N', 'Inactive',
                                                MetadataLedger.objects.
                                                values('source_metadata')
                                                )
        result_query = MetadataLedger.objects.values(
            'metadata_record_inactivation_date',
            'source_metadata_validation_status',
            'source_metadata_validation_date',
            'record_lifecycle_status').filter(
            source_metadata_key_hash=self.key_value_hash).first()

        self.assertTrue(result_query.get('source_metadata_validation_date'))
        self.assertTrue(result_query.get('metadata_record_inactivation_date'))
        self.assertEqual("N", result_query.get(
            'source_metadata_validation_status'))
        self.assertEqual('Inactive', result_query.get(
            'record_lifecycle_status'))

    # Test cases for transform_source_metadata

    def test_transform_source_using_key_overwrite(self):
        """Test to transform source metadata to target metadata schema
        format"""
        metadata_ledger = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.source_metadata_overwrite,
            source_metadata_key_hash=self.key_value_hash_overwrite,
            source_metadata_validation_status='Y',
            source_metadata_key=self.key_value_overwrite,
            source_metadata_validation_date=timezone.now(),
            source_metadata_extraction_date=timezone.now())
        metadata_ledger.save()

        test_data_dict = MetadataLedger.objects.values(
            'source_metadata').filter(
            source_metadata_validation_status='Y',
            record_lifecycle_status='Active').exclude(
            source_metadata_validation_date=None)

        test_metadata_overwrite = \
            MetadataFieldOverwrite(field_name='test_name', field_type='char',
                                   field_value='new_value', overwrite=True)
        test_metadata_overwrite.save()
        transform_source_using_key(test_data_dict, self.source_target_mapping,
                                   self.test_required_column_names,
                                   self.expected_datatype)

        result_data = MetadataLedger.objects.filter(
            source_metadata_key=self.key_value_overwrite,
            record_lifecycle_status='Active',
            source_metadata_validation_status='Y'
        ).values(
            'source_metadata_transformation_date',
            'target_metadata_key',
            'target_metadata_key_hash',
            'target_metadata',
            'target_metadata_hash').first()

        result_data_supplemental = SupplementalLedger.objects.filter(
            supplemental_metadata_key=self.key_value_overwrite,
            record_lifecycle_status='Active',
        ).values(
            'supplemental_metadata_transformation_date',
            'supplemental_metadata_key',
            'supplemental_metadata_key_hash',
            'supplemental_metadata',
            'supplemental_metadata_hash').first()

        self.assertTrue(result_data.get('source_metadata_transformation_date'))
        self.assertTrue(result_data.get('target_metadata_key'))
        self.assertTrue(result_data.get('target_metadata_key_hash'))
        self.assertTrue(result_data.get('target_metadata'))
        self.assertTrue(result_data.get('target_metadata_hash'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_transformation_date'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_key'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_key_hash'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_hash'))

    def test_transform_source_using_key(self):
        """Test to transform source metadata to target metadata schema
        format"""
        metadata_ledger = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.source_metadata,
            source_metadata_key_hash=self.key_value_hash,
            source_metadata_validation_status='Y',
            source_metadata_key=self.key_value,
            source_metadata_validation_date=timezone.now(),
            source_metadata_extraction_date=timezone.now())
        metadata_ledger.save()

        test_data_dict = MetadataLedger.objects.values(
            'source_metadata').filter(
            source_metadata_validation_status='Y',
            record_lifecycle_status='Active').exclude(
            source_metadata_validation_date=None)

        transform_source_using_key(test_data_dict, self.source_target_mapping,
                                   self.test_required_column_names,
                                   self.expected_datatype)

        result_data = MetadataLedger.objects.filter(
            source_metadata_key=self.key_value,
            record_lifecycle_status='Active',
            source_metadata_validation_status='Y'
        ).values(
            'source_metadata_transformation_date',
            'target_metadata_key',
            'target_metadata_key_hash',
            'target_metadata',
            'target_metadata_hash').first()

        result_data_supplemental = SupplementalLedger.objects.filter(
            supplemental_metadata_key=self.key_value,
            record_lifecycle_status='Active',
        ).values(
            'supplemental_metadata_transformation_date',
            'supplemental_metadata_key',
            'supplemental_metadata_key_hash',
            'supplemental_metadata',
            'supplemental_metadata_hash').first()

        self.assertTrue(result_data.get('source_metadata_transformation_date'))
        self.assertTrue(result_data.get('target_metadata_key'))
        self.assertTrue(result_data.get('target_metadata_key_hash'))
        self.assertTrue(result_data.get('target_metadata'))
        self.assertTrue(result_data.get('target_metadata_hash'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_transformation_date'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_key'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_key_hash'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata'))
        self.assertTrue(result_data_supplemental.
                        get('supplemental_metadata_hash'))

    def test_store_transformed_source_metadata(self):
        """Test to store transformed metadata and
        supplemental metadata in metadata ledger """

        metadata_ledger = MetadataLedger(
            source_metadata=self.source_metadata,
            source_metadata_key_hash=self.key_value_hash,
            source_metadata_validation_status='Y',
            record_lifecycle_status='Active',
            source_metadata_extraction_date=timezone.now()
        )
        metadata_ledger.save()

        store_transformed_source_metadata(self.key_value, self.key_value_hash,
                                          self.target_metadata,
                                          self.hash_value,
                                          self.supplemental_data
                                          )

        result_query = MetadataLedger.objects.values(
            'source_metadata_transformation_date',
            'target_metadata_key_hash',
            'target_metadata', 'target_metadata_hash').filter(
            target_metadata_key_hash=self.key_value_hash).first()

        result_query_supplemental = SupplementalLedger.objects.values(
            'supplemental_metadata_key_hash',
            'supplemental_metadata_transformation_date',
            'supplemental_metadata').filter(
            supplemental_metadata_key_hash=self.key_value_hash).first()

        self.assertTrue(result_query_supplemental.
                        get('supplemental_metadata_transformation_date'))
        self.assertEqual(self.key_value_hash, result_query_supplemental.get(
            'supplemental_metadata_key_hash'))
        self.assertEqual(self.supplemental_data, result_query_supplemental.get(
            'supplemental_metadata'))

        self.assertTrue(result_query.
                        get('source_metadata_transformation_date'))
        self.assertEqual(self.key_value_hash, result_query.get(
            'target_metadata_key_hash'))
        self.assertEqual(self.target_metadata, result_query.get(
            'target_metadata'))
        self.assertEqual(self.hash_value, result_query.get(
            'target_metadata_hash'))

    # Test cases for validate_target_metadata

    def test_get_target_validation_schema(self):
        """Test to retrieve source validation schema from XIA configuration """
        with patch('openlxp_xia.models.XIAConfiguration.field_overwrite'):
            xiaConfig = XIAConfiguration(
                target_metadata_schema='p2881_target_validation_schema.json')
            xiaConfig.save()
            result_dict = get_target_validation_schema()
            expected_dict = \
                read_json_data('p2881_target_validation_schema.json')
            self.assertEqual(expected_dict, result_dict)

    def test_validate_target_using_key(self):
        """Test for Validating target data for required columns """
        metadata_ledger = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.source_metadata,
            target_metadata=self.target_metadata,
            target_metadata_hash=self.target_hash_value,
            target_metadata_key_hash=self.target_key_value_hash,
            target_metadata_key=self.target_key_value,
            source_metadata_transformation_date=timezone.now())
        metadata_ledger.save()
        metadata_ledger_invalid = MetadataLedger(
            record_lifecycle_status='Active',
            source_metadata=self.metadata_invalid,
            target_metadata=self.target_metadata_invalid,
            target_metadata_hash=self.target_hash_value_invalid,
            target_metadata_key_hash=self.target_key_value_hash_invalid,
            target_metadata_key=self.target_key_value_invalid,
            source_metadata_transformation_date=timezone.now())
        metadata_ledger_invalid.save()
        test_data = MetadataLedger.objects.values('target_metadata_key_hash',
                                                  'target_metadata').filter(
            target_metadata_validation_status='',
            record_lifecycle_status='Active'
        ).exclude(
            source_metadata_transformation_date=None)

        validate_target_using_key(
            test_data, self.test_target_required_column_names,
            self.recommended_column_name, self.expected_datatype)
        result_query = MetadataLedger.objects.values(
            'target_metadata_validation_status', 'record_lifecycle_status'). \
            filter(target_metadata_key_hash=self.target_key_value_hash).first()

        result_query_invalid = MetadataLedger.objects.values(
            'target_metadata_validation_status', 'record_lifecycle_status'). \
            filter(target_metadata_key_hash=self.
                   target_key_value_hash_invalid).first()
        self.assertEqual('Y', result_query.get(
            'target_metadata_validation_status'))
        self.assertEqual('Active', result_query.get(
            'record_lifecycle_status'))
        self.assertEqual('N', result_query_invalid.get(
            'target_metadata_validation_status'))
        # self.assertEqual('Inactive', result_query_invalid.get(
        #     'record_lifecycle_status'))

    def test_store_target_metadata_validation_status_valid(self):
        """Test to store validation status for valid target metadata
        in metadata ledger """

        metadata_ledger = MetadataLedger(
            source_metadata=self.source_metadata,
            target_metadata=self.target_metadata,
            target_metadata_key_hash=self.key_value_hash)
        metadata_ledger.save()

        supplemental_ledger = SupplementalLedger(
            supplemental_metadata={"key": "value"},
            supplemental_metadata_key_hash=self.key_value_hash)
        supplemental_ledger.save()
        store_target_metadata_validation_status(MetadataLedger.objects.
                                                values('target_metadata'),
                                                self.key_value_hash,
                                                'Y', 'Active', MetadataLedger.
                                                objects.
                                                values('target_metadata')
                                                )
        result_query = MetadataLedger.objects.values(
            'target_metadata_validation_status',
            'target_metadata_validation_date',
            'record_lifecycle_status').filter(
            target_metadata_key_hash=self.key_value_hash).first()

        result_query_supplemental = SupplementalLedger.objects.values(
            'supplemental_metadata_validation_date').filter(
            supplemental_metadata_key_hash=self.key_value_hash).first()

        self.assertTrue(result_query_supplemental.
                        get('supplemental_metadata_validation_date'))
        self.assertEqual("Y", result_query.get(
            'target_metadata_validation_status'))
        self.assertEqual('Active', result_query.get(
            'record_lifecycle_status'))

    def test_store_target_metadata_validation_status_invalid(self):
        """Test to store validation status for invalid target metadata
        in metadata ledger """

        metadata_ledger = MetadataLedger(
            source_metadata=self.source_metadata,
            target_metadata=self.target_metadata,
            target_metadata_key_hash=self.key_value_hash)
        metadata_ledger.save()
        supplemental_ledger = SupplementalLedger(
            supplemental_metadata={"key": "value"},
            supplemental_metadata_key_hash=self.key_value_hash)
        supplemental_ledger.save()
        store_target_metadata_validation_status(MetadataLedger.objects.
                                                values('target_metadata'),
                                                self.key_value_hash,
                                                'N', 'Inactive',
                                                MetadataLedger.objects.
                                                values('target_metadata')
                                                )
        result_query = MetadataLedger.objects.values(
            'metadata_record_inactivation_date',
            'target_metadata_validation_status',
            'target_metadata_validation_date',
            'record_lifecycle_status').filter(
            target_metadata_key_hash=self.key_value_hash).first()

        result_query_supplemental = SupplementalLedger.objects.values(
            'supplemental_metadata_validation_date').filter(
            supplemental_metadata_key_hash=self.key_value_hash).first()

        self.assertTrue(result_query_supplemental.
                        get('supplemental_metadata_validation_date'))
        self.assertTrue(result_query.get('target_metadata_validation_date'))
        self.assertTrue(result_query.get('metadata_record_inactivation_date'))
        self.assertEqual("N", result_query.get(
            'target_metadata_validation_status'))
        self.assertEqual('Inactive', result_query.get(
            'record_lifecycle_status'))

    # Test cases for load_target_metadata

    def test_rename_metadata_ledger_fields(self):
        """Test for Renaming XIA column names to match with XIS column names"""
        with patch('openlxp_xia.models.XIAConfiguration.field_overwrite'):
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaConfig.save()

            return_data = rename_metadata_ledger_fields(self.xia_data)
            self.assertEquals(self.xis_expected_data['metadata_hash'],
                              return_data['metadata_hash'])
            self.assertEquals(self.xis_expected_data['metadata_key'],
                              return_data['metadata_key'])
            self.assertEquals(self.xis_expected_data['metadata_key_hash'],
                              return_data['metadata_key_hash'])
            self.assertEquals(self.xis_expected_data['provider_name'],
                              return_data['provider_name'])

    def test_post_data_to_xis_response_201(self):
        """POSTing XIA metadata_ledger to XIS metadata_ledger and receive
        response status code 201"""
        with patch('openlxp_xia.models.XIAConfiguration.field_overwrite'):
            metadata_ledger = MetadataLedger(
                record_lifecycle_status='Active',
                source_metadata=self.source_metadata,
                target_metadata=self.target_metadata,
                target_metadata_hash=self.target_hash_value,
                target_metadata_key_hash=self.target_key_value_hash,
                target_metadata_key=self.target_key_value,
                source_metadata_transformation_date=timezone.now(),
                target_metadata_validation_status='Y',
                source_metadata_validation_status='Y',
                target_metadata_transmission_status='Ready')
            metadata_ledger.save()
            input_data = MetadataLedger.objects.filter(
                record_lifecycle_status='Active',
                target_metadata_validation_status='Y',
                target_metadata_transmission_status='Ready').values(
                'metadata_record_uuid',
                'target_metadata',
                'target_metadata_hash',
                'target_metadata_key',
                'target_metadata_key_hash')
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaConfig.save()
            xisConfig = XISConfiguration(
                xis_metadata_api_endpoint=self.xis_api_endpoint_url)
            xisConfig.save()
            with patch('requests.post') as response_obj:
                response_obj.return_value = response_obj
                response_obj.status_code = 201

                post_data_to_xis(input_data)
                result_query = MetadataLedger.objects.values(
                    'target_metadata_transmission_status_code',
                    'target_metadata_transmission_status').filter(
                    target_metadata_key=self.target_key_value).first()

                self.assertEqual(201, result_query.get(
                    'target_metadata_transmission_status_code'))
                self.assertEqual('Successful', result_query.get(
                    'target_metadata_transmission_status'))

    def test_post_data_to_xis_responses_other_than_201(self):
        """POSTing XIA metadata_ledger to XIS metadata_ledger and receive
        response status code 201"""
        with patch('openlxp_xia.models.XIAConfiguration.field_overwrite'):
            metadata_ledger = MetadataLedger(
                record_lifecycle_status='Active',
                source_metadata=self.source_metadata,
                target_metadata=self.target_metadata,
                target_metadata_hash=self.target_hash_value,
                target_metadata_key_hash=self.target_key_value_hash,
                target_metadata_key=self.target_key_value,
                source_metadata_transformation_date=timezone.now(),
                target_metadata_validation_status='Y',
                source_metadata_validation_status='Y',
                target_metadata_transmission_status='Ready')
            metadata_ledger.save()
            input_data = MetadataLedger.objects.filter(
                record_lifecycle_status='Active',
                target_metadata_validation_status='Y',
                target_metadata_transmission_status='Ready').values(
                'metadata_record_uuid',
                'target_metadata',
                'target_metadata_hash',
                'target_metadata_key',
                'target_metadata_key_hash')
            xiaConfig = XIAConfiguration(publisher='AGENT')
            xiaConfig.save()
            xisConfig = XISConfiguration(
                xis_metadata_api_endpoint=self.xis_api_endpoint_url)
            xisConfig.save()
            with patch('requests.post') as response_obj:
                response_obj.return_value = response_obj
                response_obj.status_code = 400
                post_data_to_xis(input_data)
                result_query = MetadataLedger.objects.values(
                    'target_metadata_transmission_status_code',
                    'target_metadata_transmission_status').filter(
                    target_metadata_key=self.target_key_value).first()
                self.assertEqual(400, result_query.get(
                    'target_metadata_transmission_status_code'))
                self.assertEqual('Failed', result_query.get(
                    'target_metadata_transmission_status'))



from django.contrib import admin

from .models import MetadataFieldOverwrite, XIAConfiguration, XISConfiguration


def marked_default(MetadataFieldOverwriteAdmin, request, queryset):
    queryset.filter(field_type="str").update(field_value='Not Available')
    queryset.filter(field_type="datetime").\
        update(field_value='1900-01-01T00:00:00-05:00')
    queryset.filter(field_type="INT").update(field_value=0)
    queryset.filter(field_type="BOOL").update(field_value=False)


def unmarked_default(MetadataFieldOverwriteAdmin, request, queryset):
    queryset.update(field_value=None)


marked_default.short_description = "Mark default values for fields"
unmarked_default.short_description = "Unmarked default values for fields"


@admin.register(XIAConfiguration)
class XIAConfigurationAdmin(admin.ModelAdmin):
    list_display = (
        'publisher',
        'source_metadata_schema',
        'source_target_mapping',
        'target_metadata_schema',)
    fields = ['publisher',
              'source_metadata_schema',
              ('source_target_mapping',
               'target_metadata_schema')]

    def delete_queryset(self, request, queryset):
        metadata_fields = MetadataFieldOverwrite.objects.all()
        metadata_fields.delete()
        super().delete_queryset(request, queryset)


@admin.register(XISConfiguration)
class XISConfigurationAdmin(admin.ModelAdmin):
    list_display = ('xis_metadata_api_endpoint',
                    'xis_supplemental_api_endpoint',)
    fields = ['xis_metadata_api_endpoint',
              'xis_supplemental_api_endpoint']


@admin.register(MetadataFieldOverwrite)
class MetadataFieldOverwriteAdmin(admin.ModelAdmin):
    list_display = ('field_name',
                    'field_type',
                    'field_value',
                    'overwrite',)
    fields = ['field_name',
              'field_type',
              'field_value',
              'overwrite']
    actions = [marked_default, unmarked_default]




import logging

from ddt import ddt
from django.test import tag

from openlxp_xia.management.utils.xis_client import \
    get_xis_metadata_api_endpoint
from openlxp_xia.models import XISConfiguration

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('integration')
@ddt
class CommandIntegration(TestSetUp):
    # globally accessible data sets
    def test_get_xis_metadata_api_endpoint(self):
        """Test that get target mapping_dictionary from XIAConfiguration """
        xisConfig = XISConfiguration(
            xis_metadata_api_endpoint='test_api')
        xisConfig.save()
        XIS_api = get_xis_metadata_api_endpoint()
        self.assertTrue(XIS_api)



from uuid import UUID

import pandas as pd
from django.test import TestCase


class TestSetUp(TestCase):
    """Class with setup and teardown for tests in XIS"""

    def setUp(self):
        """Function to set up necessary data for testing"""

        # globally accessible data sets
        self.source_metadata = {
            "Test": "0",
            "Test_id": "2146",
            "Test_url": "https://example.test.com/",
            "End_date": "9999-12-31T00:00:00-05:00",
            "test_name": "test name",
            "Start_date": "2017-03-28T00:00:00-04:00",
            "KEY": "TestData 123",
            "SOURCESYSTEM": "AGENT",
            "test_description": "test description",
            "supplemental_data": "sample1"
        }
        self.key_value = "TestData 123_AGENT"
        self.key_value_hash = "d2a7f8cc5d5484a4dde099c6a21a903a"
        self.hash_value = "f454114ba41034e14df2a8f3c14a047d"

        self.target_metadata = {
            "Course": {
                "CourseCode": "TestData 123",
                "CourseTitle": "Acquisition Law",
                "CourseAudience": "test_data",
                "DepartmentName": "",
                "CourseObjective": "test_data",
                "CourseShortDescription": "test description",
                "CourseProviderName": "AGENT",
                "CourseSpecialNotes": "test_data",
                "CoursePrerequisites": "None",
                "EstimatedCompletionTime": "4.5 days",
                "CourseSectionDeliveryMode": "Resident",
                "CourseAdditionalInformation": "None"
            },
            "CourseInstance": {
                "CourseURL": "https://agent.tes.com/ui/lms-learning-details"
            },
            "General_Information": {
                "EndDate": 1,
                "StartDate": "1900-01-01T00:00:00-05:00"
            }
        }

        self.target_key_value = "TestData 123_AGENT"
        self.target_key_value_hash = "d2a7f8cc5d5484a4dde099c6a21a903a"
        self.target_hash_value = "eaf3e57b7f21b4d813f1258fb4ebf89d"

        self.source_metadata_overwrite = {
            "Test": "0",
            "Test_id": "2146",
            "Test_url": "https://example.test.com/",
            "End_date": "9999-12-31T00:00:00-05:00",
            "test_name": "test name",
            "Start_date": "2017-03-28T00:00:00-04:00",
            "KEY": "TestData 234",
            "SOURCESYSTEM": "AGENT",
            "test_description": "test description",
            "supplemental_data": "sample1"
        }
        self.key_value_overwrite = "TestData 234_AGENT"
        self.key_value_hash_overwrite = "5a9a682afe965fb2aa1f9f50e5148ebd"
        self.hash_value_overwrite = "81b86d6ac3b4d8561bd26f11d8feea9a"

        self.target_metadata_overwrite = {
            "Course": {
                "CourseCode": "TestData 234",
                "CourseTitle": "Acquisition Law",
                "CourseAudience": "test_data",
                "DepartmentName": "",
                "CourseObjective": "test_data",
                "CourseShortDescription": "test description",
                "CourseProviderName": "AGENT",
                "CourseSpecialNotes": "test_data",
                "CoursePrerequisites": "None",
                "EstimatedCompletionTime": "4.5 days",
                "CourseSectionDeliveryMode": "Resident",
                "CourseAdditionalInformation": "None"
            },
            "CourseInstance": {
                "CourseURL": "https://agent.tes.com/ui/lms-learning-details"
            },
            "General_Information": {
                "EndDate": "end_date",
                "StartDate": "start_date"
            }
        }

        self.target_overwrite_key_value = "TestData 234_AGENT"
        self.target_overwrite_key_value_hash = \
            "d2a7f8cc5d5484a4dde099c6a21a903a"
        self.target_overwrite_hash_value = "eaf3e57b7f21b4d813f1258fb4ebf89d"

        self.xia_data = {
            'metadata_record_uuid': UUID(
                '09edea0e-6c83-40a6-951e-2acee3e99502'),
            'target_metadata': {
                "Course": {
                    "CourseCode": "TestData 123",
                    "CourseTitle": "Acquisition Law",
                    "CourseAudience": "test_data",
                    "DepartmentName": "",
                    "CourseObjective": "test_data",
                    "CourseDescription": "test_data",
                    "CourseProviderName": "AGENT",
                    "CourseSpecialNotes": "test_data",
                    "CoursePrerequisites": "None",
                    "EstimatedCompletionTime": "4.5 days",
                    "CourseSectionDeliveryMode": "Resident",
                    "CourseAdditionalInformation": "None"
                },
                "CourseInstance": {
                    "CourseURL":
                        "https://agent.tes.com/ui/lms-learning-details"
                },
                "General_Information": {
                    "EndDate": "end_date",
                    "StartDate": "start_date"
                }
            },
            'target_metadata_hash': 'df0b51d7b45ca29682e930d236963584',
            'target_metadata_key': 'TestData 123_AGENT',
            'target_metadata_key_hash': 'd2a7f8cc5d5484a4dde099c6a21a903a'
        }
        self.xis_expected_data = {
            'unique_record_identifier': UUID(
                '09edea0e-6c83-40a6-951e-2acee3e99502'),
            'metadata': {
                "Course": {
                    "CourseCode": "TestData 123",
                    "CourseTitle": "Acquisition Law",
                    "CourseAudience": "test_data",
                    "DepartmentName": "",
                    "CourseObjective": "test_data",
                    "CourseDescription": "test_data",
                    "CourseProviderName": "AGENT",
                    "CourseSpecialNotes": "test_data",
                    "CoursePrerequisites": "None",
                    "EstimatedCompletionTime": "4.5 days",
                    "CourseSectionDeliveryMode": "Resident",
                    "CourseAdditionalInformation": "None"
                },
                "CourseInstance": {
                    "CourseURL":
                        "https://agent.tes.com/ui/lms-learning-details"
                },
                "General_Information": {
                    "EndDate": "end_date",
                    "StartDate": "start_date"
                }
            },
            'metadata_hash': 'df0b51d7b45ca29682e930d236963584',
            'metadata_key': 'TestData 123_AGENT',
            'metadata_key_hash': 'd2a7f8cc5d5484a4dde099c6a21a903a',
            'provider_name': 'AGENT'
        }

        self.xis_supplemental_expected_data = {
            'unique_record_identifier': UUID(
                '09edea0e-6c83-40a6-951e-2acee3e99502'),
            'metadata': {
                "Field1": "ABC",
                "Field2": "123",
                "Field3": "ABC-123"
            },
            'metadata_hash': 'df0b51d7b45ca29682e930d236963584',
            'metadata_key': 'TestData 123_AGENT',
            'metadata_key_hash':
                '6acf7689ea81a1f792e7668a23b1acf5',
            'provider_name': 'AGENT'
        }

        self.xia_supplemental_data = {
            'metadata_record_uuid': UUID(
                '09edea0e-6c83-40a6-951e-2acee3e99502'),
            'supplemental_metadata': {
                "Field1": "ABC",
                "Field2": "123",
                "Field3": "ABC-123"
            },
            'supplemental_metadata_hash': 'df0b51d7b45ca29682e930d236963584',
            'supplemental_metadata_key': 'TestData 123_AGENT',
            'supplemental_metadata_key_hash':
                '6acf7689ea81a1f792e7668a23b1acf5'
        }

        self.xis_supplemental_expected_data = {
            'unique_record_identifier': UUID(
                '09edea0e-6c83-40a6-951e-2acee3e99502'),
            'metadata': {
                "Field1": "ABC",
                "Field2": "123",
                "Field3": "ABC-123"
            },
            'metadata_hash': 'df0b51d7b45ca29682e930d236963584',
            'metadata_key': 'TestData 123_AGENT',
            'metadata_key_hash':
                '6acf7689ea81a1f792e7668a23b1acf5',
            'provider_name': 'AGENT'
        }

        self.metadata_invalid = {
            "Test": "0",
            "Test_id": "2146",
            "Test_url": "https://example.test.com/",
            "End_date": "9999-12-31T00:00:00-05:00",
            "test_name": "",
            "Start_date": "",
            "KEY": "TestData 1234",
            "SOURCESYSTEM": "AGENT",
            "test_description": "test description",
        }
        self.key_value_invalid = "TestData 1234_AGENT"
        self.key_value_hash_invalid = "eaf3e57b7f21b4d813f1258fb4ebf89d"
        self.hash_value_invalid = "d9eccc6651c0b95db975aca43fa9b481"

        self.target_metadata_invalid = {
            "Course": {
                "CourseCode": "TestData 1234",
                "CourseTitle": "Acquisition Law",
                "CourseAudience": "test_data",
                "DepartmentName": "",
                "CourseObjective": "test_data",
                "CourseDescription": "",
                "CourseProviderName": "AGENT",
                "CourseSpecialNotes": "test_data",
                "CoursePrerequisites": "None",
                "EstimatedCompletionTime": "4.5 days",
                "CourseSectionDeliveryMode": "Resident",
                "CourseAdditionalInformation": "None"
            },
            "CourseInstance": {
                "CourseURL": "https://agent.tes.com/ui/lms-learning-details"
            },
            "General_Information": {
                "EndDate": "end_date",
                "StartDate": "start_date"
            }
        }

        self.target_key_value_invalid = "TestData 1234_AGENT"
        self.target_key_value_hash_invalid = "d9eccc6651c0b95db975aca43fa9b481"
        self.target_hash_value_invalid = "eaf3e57b7f21b4d813f1258fb4ebf89d"

        self.source_target_mapping = {
            "Course": {
                "CourseProviderName": "SOURCESYSTEM",
                "DepartmentName": "",
                "CourseCode": "KEY",
                "CourseTitle": "test_name",
                "CourseDescription": "test_description",
                "CourseAudience": "test_attendies",
                "CourseSectionDeliveryMode": "test_mode",
                "CourseObjective": "test_objective",
                "CoursePrerequisites": "test_prerequisite",
                "EstimatedCompletionTime": "test_length",
                "CourseSpecialNotes": "test_notes",
                "CourseAdditionalInformation": "test_postscript"
            },
            "CourseInstance": {
                "CourseURL": "test_url"
            },
            "General_Information": {
                "StartDate": "start_date",
                "EndDate": "end_date"
            }
        }

        self.schema_data_dict = {
            'SOURCESYSTEM': 'Required',
            'test_id': 'Optional',
            'KEY': 'Required',
            'test_name': 'Required',
            'test_description': 'Required',
            'test_objective': 'Optional',
            'test_attendies': 'Optional',
            'test_images': 'Optional',
            'test1_id': 'Optional',
            'test_url': 'Optional',
            'Start_date.use': 'Required',
            'End_date': 'Required',
            'Test_current': 'Recommended'
        }

        self.target_data_dict = {
            'Course': {
                'CourseProviderName': {
                    'use': 'Required'
                },
                'DepartmentName': {
                    'use': 'Optional'
                },
                'CourseCode': {
                    'use': 'Required'
                },
                'CourseTitle': {
                    'use': 'Required'
                },
                'CourseDescription': {
                    'use': 'Required'
                },
                'CourseShortDescription': {
                    'use': 'Required'
                },
                'CourseFullDescription': {
                    'use': 'Optional'
                },
                'CourseAudience': {
                    'use': 'Optional'
                },
                'CourseSectionDeliveryMode': {
                    'use': 'Optional'
                },
                'CourseObjective': {
                    'use': 'Optional'
                },
                'CoursePrerequisites': {
                    'use': 'Optional'
                },
                'EstimatedCompletionTime': {
                    'use': 'Optional'
                },
                'CourseSpecialNotes': {
                    'use': 'Optional'
                },
                'CourseAdditionalInformation': {
                    'use': 'Optional'
                },
                'CourseURL': {
                    'use': 'Optional'
                },
                'CourseLevel': {
                    'use': 'Optional'
                },
                'CourseSubjectMatter': {
                    'use': 'Required'
                }
            },
            'CourseInstance': {
                'CourseCode': {
                    'use': 'Required'
                },
                'CourseTitle': {
                    'use': 'Required'
                },
                'Thumbnail': {
                    'use': 'Recommended'
                },
                'CourseShortDescription': {
                    'use': 'Optional'
                },
                'CourseFullDescription': {
                    'use': 'Optional'
                },
                'CourseURL': {
                    'use': 'Optional'
                },
                'StartDate': {
                    'use': 'Required'
                },
                'EndDate': {
                    'use': 'Required'
                },
                'EnrollmentStartDate': {
                    'use': 'Optional'
                },
                'EnrollmentEndDate': {
                    'use': 'Optional'
                },
                'DeliveryMode': {
                    'use': 'Required'
                },
                'InLanguage': {
                    'use': 'Optional'
                },
                'Instructor': {
                    'use': 'Required'
                },
                'Duration': {
                    'use': 'Optional'
                },
                'CourseLearningOutcome': {
                    'use': 'Optional'
                },
                'CourseLevel': {
                    'use': 'Optional'
                },
                'InstructorBio': {
                    'use': 'Optional'
                }
            },
            'General_Information': {
                'StartDate': {
                    'use': 'Required',
                    'data_type': 'datetime'
                },
                'EndDate': {
                    'use': 'Required',
                    'data_type': 'datetime'
                }
            },
            'Technical_Information': {
                'Thumbnail': {
                    'use': 'Recommended'
                }
            }}

        self.expected_datatype = {
            'General_Information.StartDate': 'datetime',
            'General_Information.EndDate': 'datetime'}

        self.datatype_list_as_string = {
            'key1.data_type': 'datetime',
            'key2.data_type': 'str',
            'key3.data_type': 'int',
            'key4.data_type': 'bool'
        }

        self.datatype_list_as_object = {
            'key1': 'datetime',
            'key2': str,
            'key3': int,
            'key4': bool
        }

        self.test_target_required_column_names = {
            'Course.CourseCode',
            'Course.CourseProviderName',
            'Course.CourseShortDescription'}
        self.recommended_column_name = {'Technical_Information.Thumbnail',
                                        'CourseInstance.Thumbnail'}

        self.xis_api_endpoint_url = 'http://openlxp-xis:8020/api/metadata/'

        self.supplemental_api_endpoint = 'http://openlxp-xis:8020' \
                                         '/api/supplemental-data/'

        self.test_required_column_names = ['SOURCESYSTEM',
                                           'KEY',
                                           'Start_date', 'End_date']

        self.test_metadata_column_list = [["Test", "Test_id", "Test_url"]]

        self.source_metadata_with_supplemental = {
            "Test": "0",
            "Test_id": "2146",
            "Test_url": "https://example.test.com/",
            "supplemental_data1": "sample1",
            "supplemental_data2": "sample2"
        }

        self.supplemental_data = {
            "supplemental_data1": "sample1",
            "supplemental_data2": "sample2"
        }

        self.metadata_df = pd.DataFrame.from_dict(
            {0: {1: self.source_metadata}}, orient='index')

        return super().setUp()

    def tearDown(self):
        return super().tearDown()
